
import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.Logger;


public class Test {

   DBInterface mapper; // mapper used for this test

   Test (String mapperFile) {

      // create the mapper to be used for this test
      try {
         DBModule dbm = DBModule.init (mapperFile);
         mapper = dbm.createDefaultMapper ("default");
         Logger.debug (mapper.toString());
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
         System.exit(0);
      }
      this.mapper = mapper;
   }

   public static void main (String[] args) {
      try {
         String mapperFile = args[0];
         Test testSuite = new Test(mapperFile);
         testSuite.test();
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
      }
   }

   void test() throws Exception {
      AttrValMap aValMap = null;
      HashMap    hValMap = null;

      Logger.debug ("cleanup");
      // Delete older entries
      mapper.deleteAll (Primitive.class);
      display ();

      // Create few Primitive objects and persist them to database
      Primitive o1 = new Primitive(1,null);
      Primitive o2 = new Primitive(2, "Hey");
      Primitive o3 = new Primitive(3, "Welcome !");
      mapper.create (o1);
      mapper.create (o2);

      display ();



      // Demonstrate successful transaction commit
      // Create a new transaction for current thread
      mapper.beginTransaction();
      Logger.debug ("demonstrating transaction commit.");
      try {
         // Perfrom some valid database operations
         mapper.create (o3);
         mapper.delete (o2);
         o3.setY ("New value");
         mapper.update (o3);
         // Commit all database changes
         mapper.commitTransaction();
      }
      catch (Exception ex1) {
         // Should not happen, just in case (dump the error msg and
         // terminate the process)
         Logger.debug ("failure: Unexpected exception : " + ex1);
         System.exit(1);
      }
      Logger.debug ("sucess. the values after modification.");
      display ();



      // Demonstrating unsuccessful transaction (rollback case)
      // Create a new transaction for current thread
      mapper.beginTransaction();
      Logger.debug ("demonstrating transaction rollback.");
      try {
         // Perfrom some valid database operations
         mapper.create (o2);
         mapper.delete (o3);
         // Bad operation. Trying to re-insert o1 which will fail as
         // it is a duplicate record (primary key violation)
         mapper.create (o1);
         // Code should never reach here, if it does, dump the error
         // msg and terminate the process
         Logger.debug ("failure: Unexpected error.");
         System.exit(1);
      }
      catch (Exception ex1) {
         // Rollback the database changes made in this transaction
         mapper.rollbackTransaction();
      }
      // Dump all the records and verify changes made in above transaction
      // are rolled back (e.g. o3 is not deleted)
      Logger.debug ("sucess. the values after rollback.");
      display ();
   }


   private void display () throws Exception {
      Collection c = mapper.findAll (Primitive.class);
      Logger.debug ("Entries in database: " + c.toString());
   }
}

