import java.util.*;
import com.nec.tdd.tools.dbMapper.*;
import com.nec.tdd.tools.logger.Logger;


public class Test {

   DBInterface mapper; // mapper used for this test

   Test (String mapperFile) {

      // create the mapper to be used for this test
      try {
         DBModule dbm = DBModule.init (mapperFile);
         mapper = dbm.createDefaultMapper ("default");
         Logger.debug (mapper.toString());
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
         System.exit(0);
      }
      this.mapper = mapper;
   }

   public static void main (String[] args) {
      try {
         String mapperFile = args[0];
         Test testSuite = new Test(mapperFile);
         Logger.debug ("--------------------RING 1---------------------");
         testSuite.testRing1();
         Logger.debug ("--------------------RING 2---------------------");
         testSuite.testRing2();
      }
      catch (Exception e) {
         Logger.error ("Error details: ", e);
      }
   }

   // Test Ring1 (nested attributes mapped to basic types)
   void testRing1 () throws Exception {
      // Delete older entries
      mapper.deleteAll (Ring1.class);

      // Create new entries
      Ring1 r1 = new Ring1 (1, 3, new Circle(9, new Point(1,2)));
      Ring1 r2 = new Ring1 (2, 4, new Circle(16, new Point(2,3)));
      mapper.create (r1);
      mapper.create (r2);
      Logger.debug ("create(Object) :: Object " + r1 + " and " + r2
                   + " is successfully created.");

      // Update some nested attributes
      AttrValMap aMap1 = new AttrValMap();
      aMap1.put ("x", 5);
      aMap1.put ("radius", 11);
      mapper.update (r1, aMap1, true);
      Logger.debug ("update(object,aMap,bollean) on object " + r1
                   + " is successfully excuted.");

      // Find objects by giving nested attributes
      AttrValMap aMap2 = new AttrValMap();
      aMap2.put ("y", 3);
      Collection c1 = mapper.findByAttributes (aMap2, Ring1.class);
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                     + c1.toString() + " is successfully found.");
   }


   // Test Ring2 (nested attributes mapped to nested types) 
   void testRing2 () throws Exception {
      // Delete older entries
      mapper.deleteAll (Ring2.class);

      // Create new entries
      Ring2 r1 = new Ring2 (1, 2, new Circle(32, new Point(0,0)));
      Ring2 r2 = new Ring2 (2, 2, new Circle(25, new Point(1,1)));
      mapper.create (r1);
      mapper.create (r2);
      Logger.debug ("create(Object) :: Object " + r1 + " and " + r2
                   + " is successfully created.");

      // Create a ring which has null circle (intermediate nested attributes
      // are null)
      Ring2 r3 = new Ring2 (3, 3, null);
      mapper.create(r3);
      Logger.debug ("create(Object) :: Object " + r3
                   + " is successfully created.");

      // Update some nested attributes
      AttrValMap aMap1 = new AttrValMap();
      aMap1.put ("x", 3);
      aMap1.put ("radius", 21);
      mapper.update (r1, aMap1, true);
      Logger.debug ("update(object,aMap,bollean) on object " + r1
                   + " is successfully excuted.");

      // Find objects by giving nested attributes
      AttrValMap aMap2 = new AttrValMap();
      aMap2.put ("y", 1);
      Collection c1 = mapper.findByAttributes (aMap2, Ring2.class);
      Logger.debug ("findByAttributes(aval,class) :: Collection "
                     + c1.toString() + " is successfully found.");

      // Load ring with no circle.
      Ring2 r3dup = (Ring2) mapper.findByPrimaryKey (new Integer(3),
                                                          Ring2.class);
      Logger.debug ("findByPrimaryKey(key,class) :: found object " + r3dup);
   }

}

